# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
from faker import Factory
from faker.providers import BaseProvider

# Initialize fake
fake = Factory.create()


# Create a provider for API Names (TODO: This is taken copy-paste from python-kong. Fix duplication!)
class APIInfoProvider(BaseProvider):
    def api_name(self):
        return fake.name().replace(' ', '')

    def api_path(self):
        path = fake.uri_path()
        if not path.startswith('/'):
            path = '/%s' % path
        return path


class ConsumerInfoProvider(BaseProvider):
    def consumer_name(self):
        return fake.lexify('??????')

fake.add_provider(APIInfoProvider)
fake.add_provider(ConsumerInfoProvider)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
import uuid
from abc import ABCMeta, abstractmethod

from kong.exceptions import ConflictError
from kong_admin.models import ConsumerReference, BasicAuthReference, KeyAuthReference, OAuth2Reference

from .base import KongProxySyncEngine


class ConsumerSyncEngine(KongProxySyncEngine):
    def basic_auth(self):
        return BasicAuthSyncEngine()

    def key_auth(self):
        return KeyAuthSyncEngine()

    def oauth2(self):
        return OAuth2SyncEngine()

    def get_proxy_class(self):
        return ConsumerReference

    def on_retrieve_all(self, client):
        consumers = list(client.consumers.iterate())
        for consumer_struct in consumers:
            yield consumer_struct

    def is_published(self, client, kong_id, parent_kong_id=None):
        try:
            result = client.consumers.retrieve(str(kong_id))
        except ValueError:
            return False
        return result is not None

    def on_publish(self, client, obj):
        """
        :param client: The client to interface with kong with
        :type client: kong.contract.KongAdminContract
        :param obj: The KongProxyModel to work with
        :type obj: kong_admin.models.ConsumerReference
        :rtype: uuid.UUID
        :return: The uuid of the entity that has been published
        """
        try:
            consumer_struct = client.consumers.create_or_update(
                consumer_id=obj.kong_id, username=obj.username, custom_id=obj.custom_id)
        except ConflictError:
            consumer_struct = client.consumers.update(
                username_or_id=(obj.username or obj.kong_id), username=obj.username, custom_id=obj.custom_id)
        return uuid.UUID(consumer_struct['id'])

    def on_withdraw_by_id(self, client, kong_id, parent_kong_id=None):
        client.consumers.delete(str(kong_id))

    def after_publish(self, client, obj):
        self.basic_auth().synchronize(client, BasicAuthReference.objects.filter(consumer=obj), delete=True)
        self.key_auth().synchronize(client, KeyAuthReference.objects.filter(consumer=obj), delete=True)
        self.oauth2().synchronize(client, OAuth2Reference.objects.filter(consumer=obj), delete=True)
        super(ConsumerSyncEngine, self).after_publish(client, obj)

    def before_withdraw(self, client, obj):
        for oauth2 in OAuth2Reference.objects.filter(consumer=obj):
            self.oauth2().withdraw(client, oauth2)
        for key_auth in KeyAuthReference.objects.filter(consumer=obj):
            self.key_auth().withdraw(client, key_auth)
        for basic_auth in BasicAuthReference.objects.filter(consumer=obj):
            self.basic_auth().withdraw(client, basic_auth)
        return super(ConsumerSyncEngine, self).before_withdraw(client, obj)


class ConsumerAuthSyncEngine(KongProxySyncEngine):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_auth_client(self, client, consumer_kong_id):
        """
        Returns the authentication client to use
        """

    def on_retrieve_all(self, client):
        consumers = list(client.consumers.iterate())
        for consumer_struct in consumers:
            consumer_kong_id = consumer_struct.get('id', None)
            assert consumer_kong_id is not None

            auth_list = self.get_auth_client(client, consumer_kong_id).list(size=100).get('data', None)
            assert auth_list is not None
            for auth_struct in auth_list:
                yield auth_struct

    def is_published(self, client, kong_id, parent_kong_id=None):
        try:
            result = self.get_auth_client(client, parent_kong_id).retrieve(str(kong_id))
        except ValueError:
            return False
        return result is not None

    def get_parent_object(self, obj):
        return obj.consumer

    def get_parent_key(self):
        return 'consumer_id'

    def on_withdraw_by_id(self, client, kong_id, parent_kong_id=None):
        assert kong_id is not None
        assert parent_kong_id is not None

        self.get_auth_client(client, parent_kong_id).delete(kong_id)


class BasicAuthSyncEngine(ConsumerAuthSyncEngine):
    def get_proxy_class(self):
        return BasicAuthReference

    def get_auth_client(self, client, consumer_kong_id):
        return client.consumers.basic_auth(str(consumer_kong_id))

    def on_publish(self, client, obj):
        """
        :param client: The client to interface with kong with
        :type client: kong.contract.KongAdminContract
        :param obj: The KongProxyModel to work with
        :type obj: kong_admin.models.BasicAuthReference
        :rtype: uuid.UUID
        :return: The uuid of the entity that has been published
        """
        auth_struct = self.get_auth_client(client, obj.consumer.kong_id).create_or_update(
            basic_auth_id=obj.kong_id, username=obj.username, password=obj.password)
        return uuid.UUID(auth_struct['id'])


class KeyAuthSyncEngine(ConsumerAuthSyncEngine):
    def get_proxy_class(self):
        return KeyAuthReference

    def get_auth_client(self, client, consumer_kong_id):
        return client.consumers.key_auth(str(consumer_kong_id))

    def on_publish(self, client, obj):
        """
        :param client: The client to interface with kong with
        :type client: kong.contract.KongAdminContract
        :param obj: The KongProxyModel to work with
        :type obj: kong_admin.models.KeyAuthReference
        :rtype: uuid.UUID
        :return: The uuid of the entity that has been published
        """
        auth_struct = self.get_auth_client(client, obj.consumer.kong_id).create_or_update(
            key_auth_id=obj.kong_id, key=obj.key)
        return uuid.UUID(auth_struct['id'])


class OAuth2SyncEngine(ConsumerAuthSyncEngine):
    def get_proxy_class(self):
        return OAuth2Reference

    def get_auth_client(self, client, consumer_kong_id):
        return client.consumers.oauth2(str(consumer_kong_id))

    def on_publish(self, client, obj):
        """
        :param client: The client to interface with kong with
        :type client: kong.contract.KongAdminContract
        :param obj: The KongProxyModel to work with
        :type obj: kong_admin.models.OAuth2Reference
        :rtype: uuid.UUID
        :return: The uuid of the entity that has been published
        """
        auth_struct = self.get_auth_client(client, obj.consumer.kong_id).create_or_update(
            oauth2_id=obj.kong_id, name=obj.name, redirect_uri=obj.redirect_uri, client_id=obj.client_id,
            client_secret=obj.client_secret)

        client_id = auth_struct['client_id']
        client_secret = auth_struct['client_secret']

        if obj.client_id != client_id or obj.client_secret != client_secret:
            obj.client_id = client_id
            obj.client_secret = client_secret
            self.get_proxy_class().objects.filter(id=obj.id).update(
                client_id=obj.client_id, client_secret=obj.client_secret)

        return uuid.UUID(auth_struct['id'])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
import factory

from kong_admin.models import APIReference, ConsumerReference, PluginConfigurationReference, \
    BasicAuthReference, KeyAuthReference, OAuth2Reference
from kong_admin.enums import Plugins

from .fake import fake


class APIReferenceFactory(factory.DjangoModelFactory):
    class Meta:
        model = APIReference

    upstream_url = factory.Sequence(lambda n: fake.url().encode('utf-8'))


class PluginConfigurationReferenceFactory(factory.DjangoModelFactory):
    class Meta:
        model = PluginConfigurationReference

    api = factory.SubFactory(APIReferenceFactory)
    plugin = Plugins.RATE_LIMITING
    config = {
        'second': 1
    }


class ConsumerReferenceFactory(factory.DjangoModelFactory):
    class Meta:
        model = ConsumerReference


class BasicAuthReferenceFactory(factory.DjangoModelFactory):
    class Meta:
        model = BasicAuthReference

    username = factory.Sequence(lambda n: ('%s%s' % (fake.user_name(), n)).encode('utf-8'))
    password = factory.Sequence(lambda n: fake.password(
        length=10, special_chars=True, digits=True, upper_case=True, lower_case=True).encode('utf-8'))


class KeyAuthReferenceFactory(factory.DjangoModelFactory):
    class Meta:
        model = KeyAuthReference

    key = factory.Sequence(lambda n: fake.password(
        length=40, special_chars=True, digits=True, upper_case=True, lower_case=True).encode('utf-8'))


class OAuth2ReferenceFactory(factory.DjangoModelFactory):
    class Meta:
        model = OAuth2Reference

    name = factory.Sequence(lambda n: ('%s%s' % (fake.word(), n)).encode('utf-8'))
    redirect_uri = factory.Sequence(lambda n: fake.uri().encode('utf-8'))

__all__ = [APIReferenceFactory, PluginConfigurationReferenceFactory, ConsumerReferenceFactory,
           BasicAuthReferenceFactory, KeyAuthReferenceFactory, OAuth2ReferenceFactory]



from __future__ import unicode_literals, print_function
import uuid

from django.test import TestCase

from kong_admin import models
from kong_admin import logic
from kong_admin.factory import get_kong_client
from kong_admin.enums import Plugins


from .factories import APIReferenceFactory, PluginConfigurationReferenceFactory, ConsumerReferenceFactory, \
    BasicAuthReferenceFactory, KeyAuthReferenceFactory, OAuth2ReferenceFactory
from .fake import fake


class APIReferenceLogicTestCase(TestCase):
    def setUp(self):
        self.client = get_kong_client()
        self._cleanup_api = []

    def tearDown(self):
        self.client.close()

        for api_ref in self._cleanup_api:
            self.assertTrue(isinstance(api_ref, models.APIReference))
            api_ref = models.APIReference.objects.get(id=api_ref.id)  # reloads!!
            logic.withdraw_api(self.client, api_ref)

    def test_sync_incomplete_api(self):
        # Create incomplete api_ref
        api_ref = APIReferenceFactory(upstream_url=fake.url())

        # Mark for auto cleanup
        self._cleanup_afterwards(api_ref)

        # Try to sync, expect an error
        with self.assertRaises(ValueError):
            logic.synchronize_api(self.client, api_ref)

        self.assertFalse(api_ref.synchronized)

        # Fix api_ref
        api_ref.request_host = fake.domain_name()
        api_ref.save()

        # Sync again
        logic.synchronize_api(self.client, api_ref)
        self.assertTrue(api_ref.synchronized)

        # Check kong
        result = self.client.apis.retrieve(api_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['upstream_url'], api_ref.upstream_url)
        self.assertEqual(result['request_host'], api_ref.request_host)

    def test_sync_api(self):
        # Create api_ref
        api_ref = APIReferenceFactory(upstream_url=fake.url(), request_host=fake.domain_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(api_ref)

        # Sync
        logic.synchronize_api(self.client, api_ref)
        self.assertTrue(api_ref.synchronized)

        # Check kong
        result = self.client.apis.retrieve(api_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['upstream_url'], api_ref.upstream_url)
        self.assertEqual(result['request_host'], api_ref.request_host)

    def test_sync_updated_api(self):
        # Create api_ref
        api_ref = APIReferenceFactory(upstream_url=fake.url(), request_host=fake.domain_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(api_ref)

        # Publish
        logic.synchronize_api(self.client, api_ref)
        self.assertTrue(api_ref.synchronized)

        # Check kong
        result = self.client.apis.retrieve(api_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['upstream_url'], api_ref.upstream_url)
        self.assertEqual(result['request_host'], api_ref.request_host)
        self.assertEqual(result['name'], api_ref.request_host)

        # Update
        new_name = fake.api_name()
        self.assertNotEqual(new_name, api_ref.name)
        api_ref.name = new_name
        api_ref.save()

        # Publish
        logic.synchronize_api(self.client, api_ref)
        self.assertTrue(api_ref.synchronized)

        # Check kong
        result = self.client.apis.retrieve(api_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['upstream_url'], api_ref.upstream_url)
        self.assertEqual(result['request_host'], api_ref.request_host)
        self.assertEqual(result['name'], new_name)

    def test_withdraw_api(self):
        # Create api_ref
        api_ref = APIReferenceFactory(upstream_url=fake.url(), request_host=fake.domain_name())

        # Publish
        logic.synchronize_api(self.client, api_ref)
        self.assertTrue(api_ref.synchronized)

        # Check kong
        result = self.client.apis.retrieve(api_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['upstream_url'], api_ref.upstream_url)
        self.assertEqual(result['request_host'], api_ref.request_host)

        # Store kong_id
        kong_id = api_ref.kong_id

        # You can delete afterwards
        logic.withdraw_api(self.client, api_ref)
        self.assertFalse(api_ref.synchronized)

        # Check kong
        with self.assertRaises(ValueError):
            _ = self.client.apis.retrieve(kong_id)

    def test_delete_api(self):
        # Create api_ref
        api_ref = APIReferenceFactory(upstream_url=fake.url(), request_host=fake.domain_name())

        # Publish
        logic.synchronize_api(self.client, api_ref)
        self.assertTrue(api_ref.synchronized)

        # Check kong
        result = self.client.apis.retrieve(api_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['upstream_url'], api_ref.upstream_url)
        self.assertEqual(result['request_host'], api_ref.request_host)

        # You can delete afterwards
        api_kong_id = api_ref.kong_id
        api_ref.delete()

        # Check kong
        with self.assertRaises(ValueError):
            _ = self.client.apis.retrieve(api_kong_id)

    def test_sync_plugin_configuration_before_api(self):
        # Create api_ref
        api_ref = APIReferenceFactory(upstream_url=fake.url(), request_host=fake.domain_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(api_ref)

        # Create plugin_configuration
        plugin_configuration_ref = PluginConfigurationReferenceFactory(api=api_ref)

        # Attempt to publish
        with self.assertRaises(ValueError):
            logic.synchronize_plugin_configuration(self.client, plugin_configuration_ref)

    def test_sync_plugin_configuration_without_fields(self):
        # Create api_ref
        api_ref = APIReferenceFactory(upstream_url=fake.url(), request_host=fake.domain_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(api_ref)

        # Publish api
        logic.synchronize_api(self.client, api_ref)

        # Check
        result = self.client.apis.retrieve(api_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['upstream_url'], api_ref.upstream_url)

        # Create plugin_configuration
        plugin_configuration_ref = PluginConfigurationReferenceFactory(api=api_ref, config={})

        # Attempt to publish
        with self.assertRaises(ValueError):
            logic.synchronize_plugin_configuration(self.client, plugin_configuration_ref)

    def test_sync_plugin_configuration(self):
        # Create api_ref
        api_ref = APIReferenceFactory(upstream_url=fake.url(), request_host=fake.domain_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(api_ref)

        # Publish api
        logic.synchronize_api(self.client, api_ref)

        # Check
        result = self.client.apis.retrieve(api_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['upstream_url'], api_ref.upstream_url)

        # Create plugin_configuration
        plugin_configuration_ref = PluginConfigurationReferenceFactory(api=api_ref)

        # Publish plugin_configuration
        logic.synchronize_plugin_configuration(self.client, plugin_configuration_ref)

        # Check
        result = self.client.apis.plugins(api_ref.kong_id).retrieve(plugin_configuration_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['name'], Plugins.label(plugin_configuration_ref.plugin))

    def test_withdraw_plugin_configuration(self):
        # Create api_ref
        api_ref = APIReferenceFactory(upstream_url=fake.url(), request_host=fake.domain_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(api_ref)

        # Publish api
        logic.synchronize_api(self.client, api_ref)

        # Create plugin_configuration
        plugin_configuration_ref = PluginConfigurationReferenceFactory(api=api_ref)

        # Publish plugin_configuration
        logic.synchronize_plugin_configuration(self.client, plugin_configuration_ref)

        # Check
        result = self.client.apis.plugins(api_ref.kong_id).retrieve(plugin_configuration_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['name'], Plugins.label(plugin_configuration_ref.plugin))

        # Withdraw plugin_configuration
        logic.withdraw_plugin_configuration(self.client, plugin_configuration_ref)

        # Check
        with self.assertRaises(ValueError):
            _ = self.client.apis.plugins(api_ref.kong_id).retrieve(plugin_configuration_ref.kong_id)

    def test_delete_synchronized_plugin_configuration(self):
        # Create api_ref
        api_ref = APIReferenceFactory(upstream_url=fake.url(), request_host=fake.domain_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(api_ref)

        # Publish api
        logic.synchronize_api(self.client, api_ref)

        # Create plugin_configuration
        plugin_configuration_ref = PluginConfigurationReferenceFactory(api=api_ref)

        # Publish plugin_configuration
        logic.synchronize_plugin_configuration(self.client, plugin_configuration_ref)

        # Check
        result = self.client.apis.plugins(api_ref.kong_id).retrieve(plugin_configuration_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['name'], Plugins.label(plugin_configuration_ref.plugin))

        # Delete plugin_configuration
        plugin_configuration_kong_id = plugin_configuration_ref.kong_id
        plugin_configuration_ref.delete()

        # Check
        with self.assertRaises(ValueError):
            _ = self.client.apis.plugins(api_ref.kong_id).retrieve(plugin_configuration_kong_id)

    def test_disable_synchronized_plugin_configuration(self):
        # Create api_ref
        api_ref = APIReferenceFactory(upstream_url=fake.url(), request_host=fake.domain_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(api_ref)

        # Publish api
        logic.synchronize_api(self.client, api_ref)

        # Create plugin_configuration
        plugin_configuration_ref = PluginConfigurationReferenceFactory(api=api_ref)

        # Publish plugin_configuration
        logic.synchronize_plugin_configuration(self.client, plugin_configuration_ref)

        # Check
        result = self.client.apis.plugins(api_ref.kong_id).retrieve(plugin_configuration_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['name'], Plugins.label(plugin_configuration_ref.plugin))
        self.assertTrue(result['enabled'])

        # Update plugin_configuration
        logic.enable_plugin_configuration(self.client, plugin_configuration_ref, enabled=False)

        # Check
        result = self.client.apis.plugins(api_ref.kong_id).retrieve(plugin_configuration_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['name'], Plugins.label(plugin_configuration_ref.plugin))
        self.assertFalse(result['enabled'])

    def test_update_synchronized_plugin_configuration(self):
        # Create api_ref
        api_ref = APIReferenceFactory(upstream_url=fake.url(), request_host=fake.domain_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(api_ref)

        # Publish api
        logic.synchronize_api(self.client, api_ref)

        # Create plugin_configuration
        plugin_configuration_ref = PluginConfigurationReferenceFactory(api=api_ref)

        # Publish plugin_configuration
        logic.synchronize_plugin_configuration(self.client, plugin_configuration_ref)

        # Check
        result = self.client.apis.plugins(api_ref.kong_id).retrieve(plugin_configuration_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['name'], Plugins.label(plugin_configuration_ref.plugin))
        self.assertEqual(result['config']['second'], plugin_configuration_ref.config['second'])

        # Update plugin_configuration
        new_value = 5
        self.assertNotEqual(new_value, plugin_configuration_ref.config['second'])
        plugin_configuration_ref.config['second'] = new_value
        plugin_configuration_ref.save()
        logic.publish_plugin_configuration(self.client, plugin_configuration_ref)

        # Check
        result = self.client.apis.plugins(api_ref.kong_id).retrieve(plugin_configuration_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['name'], Plugins.label(plugin_configuration_ref.plugin))
        self.assertEqual(result['config']['second'], plugin_configuration_ref.config['second'])

    def _cleanup_afterwards(self, api_ref):
        self._cleanup_api.append(api_ref)
        return api_ref


class ConsumerReferenceLogicTestCase(TestCase):
    def setUp(self):
        self.client = get_kong_client()
        self._cleanup_consumers = []

    def tearDown(self):
        self.client.close()

        for consumer_ref in self._cleanup_consumers:
            self.assertTrue(isinstance(consumer_ref, models.ConsumerReference))
            consumer_ref = models.ConsumerReference.objects.get(id=consumer_ref.id)  # reloads!!
            logic.withdraw_consumer(self.client, consumer_ref)

    def test_incomplete_consumer(self):
        # Create incomplete consumer_ref
        consumer_ref = ConsumerReferenceFactory()

        # Mark for auto cleanup
        self._cleanup_afterwards(consumer_ref)

        # Try to sync, expect an error
        with self.assertRaises(ValueError):
            logic.synchronize_consumer(self.client, consumer_ref)

        self.assertFalse(consumer_ref.synchronized)

        # Fix consumer_ref
        consumer_ref.username = fake.consumer_name()
        consumer_ref.save()

        # Sync again
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check kong
        result = self.client.consumers.retrieve(consumer_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['username'], consumer_ref.username)

    def test_sync_consumer(self):
        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(consumer_ref)

        # Sync
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check kong
        result = self.client.consumers.retrieve(consumer_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['username'], consumer_ref.username)

    def test_sync_updated_consumer(self):
        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(consumer_ref)

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check kong
        result = self.client.consumers.retrieve(consumer_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['username'], consumer_ref.username)

        # Update
        new_name = fake.consumer_name()
        self.assertNotEqual(new_name, consumer_ref.username)
        consumer_ref.username = new_name
        consumer_ref.save()

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check kong
        result = self.client.consumers.retrieve(consumer_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['username'], new_name)

    def test_withdraw_consumer(self):
        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check kong
        result = self.client.consumers.retrieve(consumer_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['username'], consumer_ref.username)

        # Store kong_id
        kong_id = consumer_ref.kong_id

        # You can delete afterwards
        logic.withdraw_consumer(self.client, consumer_ref)
        self.assertFalse(consumer_ref.synchronized)

        # Check kong
        with self.assertRaises(ValueError):
            _ = self.client.consumers.retrieve(kong_id)

    def test_delete_consumer(self):
        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check kong
        result = self.client.consumers.retrieve(consumer_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['username'], consumer_ref.username)

        # You can delete afterwards
        consumer_kong_id = consumer_ref.kong_id
        consumer_ref.delete()

        # Check kong
        with self.assertRaises(ValueError):
            _ = self.client.consumers.retrieve(consumer_kong_id)

    def test_sync_consumer_basic_auth(self):
        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(consumer_ref)

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check kong
        amount = self.client.consumers.basic_auth(consumer_ref.kong_id).count()
        self.assertEqual(amount, 0)

        # Create auth
        auth_ref = BasicAuthReferenceFactory(consumer=consumer_ref)

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Reload
        auth_ref = models.BasicAuthReference.objects.get(id=auth_ref.id)
        self.assertIsNotNone(auth_ref.kong_id)

        # Check kong
        result = self.client.consumers.basic_auth(consumer_ref.kong_id).retrieve(auth_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['username'], auth_ref.username)
        self.assertIsNotNone(result['password'])

    def test_sync_consumer_multiple_basic_auth(self):
        amount = 3

        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(consumer_ref)

        # Create auths
        auths = []
        for i in range(amount):
            auths.append(BasicAuthReferenceFactory(consumer=consumer_ref))

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check
        self.assertEqual(self.client.consumers.basic_auth(consumer_ref.kong_id).count(), amount)

        # Reload
        for i in range(len(auths)):
            auths[i] = models.BasicAuthReference.objects.get(id=auths[i].id)
            self.assertIsNotNone(auths[i].kong_id)

        # Check kong
        result = self.client.consumers.basic_auth(consumer_ref.kong_id).list()
        self.assertIsNotNone(result)
        self.assertEqual(
            sorted([(uuid.UUID(r['id']), r['username']) for r in result['data']], key=lambda x: x[0]),
            sorted([(obj.kong_id, obj.username) for obj in auths], key=lambda x: x[0]))

    def test_withdraw_consumer_basic_auth(self):
        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Create auth
        auth_ref = BasicAuthReferenceFactory(consumer=consumer_ref)

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Reload
        auth_ref = models.BasicAuthReference.objects.get(id=auth_ref.id)
        self.assertIsNotNone(auth_ref.kong_id)
        self.assertTrue(auth_ref.synchronized)

        # Withdraw
        logic.withdraw_consumer(self.client, consumer_ref)
        self.assertFalse(consumer_ref.synchronized)

        # Reload
        auth_ref = models.BasicAuthReference.objects.get(id=auth_ref.id)
        self.assertIsNone(auth_ref.kong_id)
        self.assertFalse(auth_ref.synchronized)

    def test_delete_consumer_basic_auth(self):
        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Create auth
        auth_ref1 = BasicAuthReferenceFactory(consumer=consumer_ref)
        BasicAuthReferenceFactory(consumer=consumer_ref)

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check
        self.assertEqual(self.client.consumers.basic_auth(consumer_ref.kong_id).count(), 2)

        # Delete auth_ref1
        auth_ref1.delete()

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check
        self.assertEqual(self.client.consumers.basic_auth(consumer_ref.kong_id).count(), 1)

        # Delete consumer
        consumer_kong_id = consumer_ref.kong_id
        consumer_ref.delete()

        # Check
        with self.assertRaises(ValueError):
            self.client.consumers.basic_auth(consumer_kong_id).count()

    def test_sync_consumer_key_auth(self):
        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(consumer_ref)

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check kong
        amount = self.client.consumers.key_auth(consumer_ref.kong_id).count()
        self.assertEqual(amount, 0)

        # Create auth
        auth_ref = KeyAuthReferenceFactory(consumer=consumer_ref)

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Reload
        auth_ref = models.KeyAuthReference.objects.get(id=auth_ref.id)
        self.assertIsNotNone(auth_ref.kong_id)

        # Check kong
        result = self.client.consumers.key_auth(consumer_ref.kong_id).retrieve(auth_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['key'], auth_ref.key)

    def test_sync_consumer_multiple_key_auth(self):
        amount = 3

        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(consumer_ref)

        # Create auths
        auths = []
        for i in range(amount):
            auths.append(KeyAuthReferenceFactory(consumer=consumer_ref))

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check
        self.assertEqual(self.client.consumers.key_auth(consumer_ref.kong_id).count(), amount)

        # Reload
        for i in range(len(auths)):
            auths[i] = models.KeyAuthReference.objects.get(id=auths[i].id)
            self.assertIsNotNone(auths[i].kong_id)

        # Check kong
        result = self.client.consumers.key_auth(consumer_ref.kong_id).list()
        self.assertIsNotNone(result)
        self.assertEqual(
            sorted([(uuid.UUID(r['id']), r['key']) for r in result['data']], key=lambda x: x[0]),
            sorted([(obj.kong_id, obj.key) for obj in auths], key=lambda x: x[0]))

    def test_withdraw_consumer_key_auth(self):
        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Create auth
        auth_ref = KeyAuthReferenceFactory(consumer=consumer_ref)

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Reload
        auth_ref = models.KeyAuthReference.objects.get(id=auth_ref.id)
        self.assertIsNotNone(auth_ref.kong_id)
        self.assertTrue(auth_ref.synchronized)

        # Withdraw
        logic.withdraw_consumer(self.client, consumer_ref)
        self.assertFalse(consumer_ref.synchronized)

        # Reload
        auth_ref = models.KeyAuthReference.objects.get(id=auth_ref.id)
        self.assertIsNone(auth_ref.kong_id)
        self.assertFalse(auth_ref.synchronized)

    def test_delete_consumer_key_auth(self):
        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Create auth
        auth_ref1 = KeyAuthReferenceFactory(consumer=consumer_ref)
        KeyAuthReferenceFactory(consumer=consumer_ref)

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check
        self.assertEqual(self.client.consumers.key_auth(consumer_ref.kong_id).count(), 2)

        # Delete auth_ref1
        auth_ref1.delete()

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check
        self.assertEqual(self.client.consumers.key_auth(consumer_ref.kong_id).count(), 1)

        # Delete consumer
        consumer_kong_id = consumer_ref.kong_id
        consumer_ref.delete()

        # Check
        with self.assertRaises(ValueError):
            self.client.consumers.key_auth(consumer_kong_id).count()

    def test_sync_consumer_oauth2(self):
        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(consumer_ref)

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check kong
        amount = self.client.consumers.oauth2(consumer_ref.kong_id).count()
        self.assertEqual(amount, 0)

        # Create auth
        auth_ref = OAuth2ReferenceFactory(consumer=consumer_ref)

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Reload
        auth_ref = models.OAuth2Reference.objects.get(id=auth_ref.id)
        self.assertIsNotNone(auth_ref.kong_id)

        # Check kong
        result = self.client.consumers.oauth2(consumer_ref.kong_id).retrieve(auth_ref.kong_id)
        self.assertIsNotNone(result)
        self.assertEqual(result['client_id'], auth_ref.client_id)

    def test_sync_consumer_multiple_oauth2(self):
        amount = 3

        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Mark for auto cleanup
        self._cleanup_afterwards(consumer_ref)

        # Create auths
        auths = []
        for i in range(amount):
            auths.append(OAuth2ReferenceFactory(consumer=consumer_ref))

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check
        self.assertEqual(self.client.consumers.oauth2(consumer_ref.kong_id).count(), amount)

        # Reload
        for i in range(len(auths)):
            auths[i] = models.OAuth2Reference.objects.get(id=auths[i].id)
            self.assertIsNotNone(auths[i].kong_id)

        # Check kong
        result = self.client.consumers.oauth2(consumer_ref.kong_id).list()
        self.assertIsNotNone(result)
        self.assertEqual(
            sorted([(uuid.UUID(r['id']), r['client_id']) for r in result['data']], key=lambda x: x[0]),
            sorted([(obj.kong_id, obj.client_id) for obj in auths], key=lambda x: x[0]))

    def test_withdraw_consumer_oauth2(self):
        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Create auth
        auth_ref = OAuth2ReferenceFactory(consumer=consumer_ref)

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Reload
        auth_ref = models.OAuth2Reference.objects.get(id=auth_ref.id)
        self.assertIsNotNone(auth_ref.kong_id)
        self.assertTrue(auth_ref.synchronized)

        # Withdraw
        logic.withdraw_consumer(self.client, consumer_ref)
        self.assertFalse(consumer_ref.synchronized)

        # Reload
        auth_ref = models.OAuth2Reference.objects.get(id=auth_ref.id)
        self.assertIsNone(auth_ref.kong_id)
        self.assertFalse(auth_ref.synchronized)

    def test_delete_consumer_oauth2(self):
        # Create consumer_ref
        consumer_ref = ConsumerReferenceFactory(username=fake.consumer_name())

        # Create auth
        auth_ref1 = OAuth2ReferenceFactory(consumer=consumer_ref)
        OAuth2ReferenceFactory(consumer=consumer_ref)

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check
        self.assertEqual(self.client.consumers.oauth2(consumer_ref.kong_id).count(), 2)

        # Delete auth_ref1
        auth_ref1.delete()

        # Publish
        logic.synchronize_consumer(self.client, consumer_ref)
        self.assertTrue(consumer_ref.synchronized)

        # Check
        self.assertEqual(self.client.consumers.oauth2(consumer_ref.kong_id).count(), 1)

        # Delete consumer
        consumer_kong_id = consumer_ref.kong_id
        consumer_ref.delete()

        # Check
        with self.assertRaises(ValueError):
            self.client.consumers.oauth2(consumer_kong_id).count()

    def _cleanup_afterwards(self, consumer_ref):
        self._cleanup_consumers.append(consumer_ref)
        return consumer_ref



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
from django.apps import AppConfig


class KongAdminConfig(AppConfig):
    name = 'kong_admin'
    verbose_name = 'Kong'

    def ready(self):
        from kong_admin.receivers import __all__



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
import copy
from django.conf.urls import patterns, url
from django.contrib.admin import ModelAdmin
from django.utils.safestring import mark_safe


class CustomModelAdmin(ModelAdmin):
    change_list_template = 'kong_admin/custom_admin_change_list.html'

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context['buttons'] = getattr(self, 'action_buttons', [])
        return super(CustomModelAdmin, self).changelist_view(request, extra_context=extra_context)

    def get_urls(self):
        urls = super(CustomModelAdmin, self).get_urls()
        action_buttons = getattr(self, 'action_buttons', [])
        list_display_buttons = getattr(self, 'list_display_buttons', [])

        custom_urls = []

        for button in action_buttons:
            custom_urls.append(url(r"^%s$" % button['url'], button['view']))

        for button in list_display_buttons:
            # Add pk attribute regex that will always be provided
            button_url = CustomModelAdmin._safe_list_display_button_url(button['url']) + r'(?P<pk>\d+)/'
            custom_urls.append(url(r"^%s$" % button_url, button['view']))

        my_urls = patterns("", *custom_urls)
        return my_urls + urls

    def get_list_display(self, request):
        list_display_buttons = getattr(self, 'list_display_buttons', [])
        if not list_display_buttons:
            return self.list_display
        return self.list_display + (self._item_actions,)

    def _item_actions(self, obj):
        result = ''
        for button in getattr(self, 'list_display_buttons', []):
            result += CustomModelAdmin._render_list_display_button(obj, button)
        return mark_safe(result)
    _item_actions.short_description = 'Item Actions'

    @staticmethod
    def _render_list_display_button(obj, button):
        button = copy.copy(button)
        for key in ('url', 'caption'):
            if callable(button[key]):
                button[key] = button[key](obj)

        # Add pk attribute that will always be provided
        button['url'] = '%s%d/' % (CustomModelAdmin._safe_list_display_button_url(button['url']), obj.pk)

        # Return rendered button
        return mark_safe('<a href="%(url)s"><input type="button" value="%(caption)s" /></a>' % button)

    @staticmethod
    def _safe_list_display_button_url(button_url):
        if not button_url.endswith('/'):
            button_url += '/'
        return button_url



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
from django_enumfield import enum


class Plugins(enum.Enum):
    SSL = 1
    KEY_AUTHENTICATION = 2
    BASIC_AUTHENTICATION = 3
    OAUTH2_AUTHENTICATION = 4
    RATE_LIMITING = 5
    TCP_LOG = 6
    UDP_LOG = 7
    FILE_LOG = 8
    HTTP_LOG = 9
    CORS = 10
    REQUEST_TRANSFORMER = 11
    REPONSE_TRANSFORMER = 12
    REQUEST_SIZE_LIMITING = 13
    RESPONSE_RATE_LIMITING = 14
    IP_RESTRICTION = 15
    ACL = 16
    JWT = 17

    MASHAPE_ANALYTICS = 99

    labels = {
        SSL: 'ssl',
        KEY_AUTHENTICATION: 'key-auth',
        BASIC_AUTHENTICATION: 'basic-auth',
        OAUTH2_AUTHENTICATION: 'oauth2',
        RATE_LIMITING: 'rate-limiting',
        TCP_LOG: 'tcp-log',
        UDP_LOG: 'udp-log',
        FILE_LOG: 'file-log',
        HTTP_LOG: 'http-log',
        CORS: 'cors',
        REQUEST_TRANSFORMER: 'request-transformer',
        REPONSE_TRANSFORMER: 'response-transformer',
        REQUEST_SIZE_LIMITING: 'request-size-limiting',
        RESPONSE_RATE_LIMITING: 'response-ratelimiting',
        IP_RESTRICTION: 'ip-restriction',
        ACL: 'acl',
        JWT: 'jwt',
        MASHAPE_ANALYTICS: 'mashape-analytics'
    }



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
from .base import *

DATABASES = {
    # https://github.com/joke2k/django-environ
    #  Raises ImproperlyConfigured exception if DATABASE_URL not in os.environ, and no default
    #  was given.
    'default': env.db(default='sqlite:///%s' % os.path.join((BASE_DIR - 2).root, 'db.sqlite3'))
}

KONG_ADMIN_URL = env('KONG_ADMIN_URL', default='http://localhost:8001')
KONG_ADMIN_SIMULATOR = env('KONG_ADMIN_SIMULATOR', default=False)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'null': {
            'level': 'DEBUG',
            'class': 'logging.NullHandler',
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
    },
    'loggers': {
        'django': {
            'handlers': ['console'],
            'level': env('DJANGO_LOG_LEVEL', default='INFO'),
            'propagate': True,
        },
        'kong_admin': {
            'handlers': ['console'],
            'level': env('DJANGO_LOG_LEVEL', default='INFO'),
        },
    },
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kong_admin', '0004_auto_20150909_0826'),
    ]

    operations = [
        migrations.RenameField(
            model_name='apireference',
            old_name='inbound_dns',
            new_name='request_host',
        ),
        migrations.RenameField(
            model_name='apireference',
            old_name='path',
            new_name='request_path',
        ),
        migrations.RemoveField(
            model_name='apireference',
            name='strip_path',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import jsonfield2.fields


class Migration(migrations.Migration):

    dependencies = [
        ('kong_admin', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='pluginconfigurationfield',
            name='configuration',
        ),
        migrations.AddField(
            model_name='pluginconfigurationreference',
            name='value',
            field=jsonfield2.fields.JSONField(default={}),
        ),
        migrations.DeleteModel(
            name='PluginConfigurationField',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kong_admin', '0002_auto_20150827_0931'),
    ]

    operations = [
        migrations.AddField(
            model_name='pluginconfigurationreference',
            name='plugin',
            field=models.IntegerField(default=12),
        ),
        migrations.AlterUniqueTogether(
            name='pluginconfigurationreference',
            unique_together=set([('plugin', 'api')]),
        ),
        migrations.RemoveField(
            model_name='pluginconfigurationreference',
            name='name',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function

from .factory import get_api_sync_engine, get_consumer_sync_engine
from .models import APIReference, ConsumerReference, PluginConfigurationReference


def publish_api(client, obj):
    obj = get_api_sync_engine().publish(client, obj)
    if not obj.enabled:
        obj.enabled = True
        APIReference.objects.filter(id=obj.id).update(enabled=obj.enabled)
    return obj


def withdraw_api(client, obj):
    obj = get_api_sync_engine().withdraw(client, obj)
    if obj.enabled:
        obj.enabled = False
        APIReference.objects.filter(id=obj.id).update(enabled=obj.enabled)
    return obj


def synchronize_api(client, obj, toggle=False):
    if (toggle and obj.enabled) or (not toggle and not obj.enabled):
        return withdraw_api(client, obj)
    return publish_api(client, obj)


def synchronize_apis(client, queryset=None):
    return get_api_sync_engine().synchronize(client, queryset=queryset, delete=True)


def publish_plugin_configuration(client, obj):
    obj = get_api_sync_engine().plugins().publish(client, obj)
    return obj


def withdraw_plugin_configuration(client, obj):
    obj = get_api_sync_engine().plugins().withdraw(client, obj)
    return obj


def enable_plugin_configuration(client, obj, enabled=True):
    obj.enabled = enabled
    obj = get_api_sync_engine().plugins().publish(client, obj)

    # Updated enabled state without triggering another save
    PluginConfigurationReference.objects.filter(id=obj.id).update(enabled=obj.enabled)

    return obj


def synchronize_plugin_configuration(client, obj, toggle=False):
    enabled = not obj.enabled if toggle else obj.enabled
    return enable_plugin_configuration(client, obj, enabled=enabled)


def synchronize_plugin_configurations(client, queryset=None):
    return get_api_sync_engine().plugins().synchronize(client, queryset=queryset, delete=True)


def publish_consumer(client, obj):
    obj = get_consumer_sync_engine().publish(client, obj)
    if not obj.enabled:
        obj.enabled = True
        ConsumerReference.objects.filter(id=obj.id).update(enabled=obj.enabled)


def withdraw_consumer(client, obj):
    obj = get_consumer_sync_engine().withdraw(client, obj)
    if obj.enabled:
        obj.enabled = False
        ConsumerReference.objects.filter(id=obj.id).update(enabled=obj.enabled)


def synchronize_consumer(client, obj, toggle=False):
    if (toggle and obj.enabled) or (not toggle and not obj.enabled):
        return withdraw_consumer(client, obj)
    return publish_consumer(client, obj)


def synchronize_consumers(client, queryset=None):
    return get_consumer_sync_engine().synchronize(client, queryset=queryset, delete=True)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
from django.conf.urls import include, url
from django.contrib import admin


urlpatterns = [
    url(r'^admin/', include(admin.site.urls)),

    #Debug
    url(r'^kongconfig/', 'kong_admin.views.show_config')
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
from kong.exceptions import ConflictError
from kong_admin.models import APIReference, PluginConfigurationReference
from kong_admin.enums import Plugins

from .base import KongProxySyncEngine


class APISyncEngine(KongProxySyncEngine):
    def plugins(self):
        return PluginConfigurationSyncEngine()

    def get_proxy_class(self):
        return APIReference

    def on_retrieve_all(self, client):
        apis = list(client.apis.iterate())
        for api_struct in apis:
            yield api_struct

    def is_published(self, client, kong_id, parent_kong_id=None):
        try:
            result = client.apis.retrieve(str(kong_id))
        except ValueError:
            return False
        return result is not None

    def on_publish(self, client, obj):
        try:
            api_struct = client.apis.add_or_update(
                api_id=obj.kong_id, upstream_url=obj.upstream_url, name=obj.name, request_host=obj.request_host,
                request_path=obj.request_path)
        except ConflictError:
            api_struct = client.apis.update(
                name_or_id=(obj.name or obj.request_host), upstream_url=obj.upstream_url, name=obj.name,
                request_host=obj.request_host, request_path=obj.request_path)

        name = api_struct['name']

        if obj.name != name:
            obj.name = name
            self.get_proxy_class().objects.filter(id=obj.id).update(name=obj.name)

        return api_struct['id']

    def on_withdraw_by_id(self, client, kong_id, parent_kong_id=None):
        client.apis.delete(str(kong_id))

    def after_publish(self, client, obj):
        self.plugins().synchronize(client, PluginConfigurationReference.objects.filter(api=obj), delete=True)
        super(APISyncEngine, self).after_publish(client, obj)

    def before_withdraw(self, client, obj):
        for plugin in PluginConfigurationReference.objects.filter(api=obj):
            self.plugins().withdraw(client, plugin)
        return super(APISyncEngine, self).before_withdraw(client, obj)


class PluginConfigurationSyncEngine(KongProxySyncEngine):
    def get_proxy_class(self):
        return PluginConfigurationReference

    def on_retrieve_all(self, client):
        apis = list(client.apis.iterate())
        for api_struct in apis:
            api_kong_id = api_struct.get('id', None)
            assert api_kong_id is not None

            plugin_configurations = client.apis.plugins(api_kong_id).list(size=100).get('data', None)
            assert plugin_configurations is not None
            for plugin_configuration_struct in plugin_configurations:
                yield plugin_configuration_struct

    def is_published(self, client, kong_id, parent_kong_id=None):
        try:
            result = client.apis.plugins(str(parent_kong_id)).retrieve(str(kong_id))
        except ValueError:
            return False
        return result is not None

    def get_parent_object(self, obj):
        return obj.api

    def get_parent_key(self):
        return 'api_id'

    def on_publish(self, client, obj):
        api_kong_id = obj.api.kong_id
        consumer_kong_id = obj.consumer.kong_id if obj.consumer is not None else None

        try:
            plugin_configuration_struct = client.apis.plugins(str(api_kong_id)).create_or_update(
                plugin_configuration_id=obj.kong_id, plugin_name=Plugins.label(obj.plugin), enabled=obj.enabled,
                consumer_id=consumer_kong_id, **obj.config)
        except ConflictError:
            plugin_configuration_struct = client.apis.plugins(str(api_kong_id)).update(
                plugin_id=obj.name, enabled=obj.enabled, consumer_id=consumer_kong_id, **obj.config)

        config = plugin_configuration_struct['config']

        if obj.config != config:
            obj.config = config
            self.get_proxy_class().objects.filter(id=obj.id).update(config=obj.config)

        return plugin_configuration_struct['id']

    def on_withdraw_by_id(self, client, kong_id, parent_kong_id=None):
        assert kong_id is not None
        assert parent_kong_id is not None

        client.apis.plugins(parent_kong_id).delete(kong_id)



from __future__ import unicode_literals
from django.template import Library

register = Library()

@register.inclusion_tag('kong_admin/custom_admin_actions.html', takes_context=True)
def custom_admin_actions(context):
    """
    Track the number of times the action field has been rendered on the page,
    so we know which value to use.
    """
    context['action_index'] = context.get('action_index', -1) + 1
    return context



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kong_admin', '0003_auto_20150909_0756'),
    ]

    operations = [
        migrations.RenameField(
            model_name='pluginconfigurationreference',
            old_name='value',
            new_name='config',
        ),
        migrations.AlterField(
            model_name='pluginconfigurationreference',
            name='plugin',
            field=models.IntegerField(default=13),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
import json
from django.conf import settings
from django import forms
from django.shortcuts import render
from django.http.response import HttpResponse

from .factory import get_kong_client


def show_config(request):
    """
    Debug view, this should not be used in production!
    This view shows the configuration as it is known by Kong

    :param request:
    :return:
    """
    if not settings.DEBUG:
        return HttpResponse(status=403)

    kong = get_kong_client()

    result = {
        'apis': list(kong.apis.iterate()),
        'consumers': list(kong.consumers.iterate()),
    }

    for i in range(len(result['apis'])):
        result['apis'][i]['plugins'] = list(kong.apis.plugins(result['apis'][i]['id']).iterate()),

    for i in range(len(result['consumers'])):
        result['consumers'][i]['basicauth'] = list(kong.consumers.basic_auth(result['consumers'][i]['id']).iterate()),
        result['consumers'][i]['keyauth'] = list(kong.consumers.key_auth(result['consumers'][i]['id']).iterate()),
        result['consumers'][i]['oauth2'] = list(kong.consumers.oauth2(result['consumers'][i]['id']).iterate()),

    config = json.dumps(result, sort_keys=True, indent=4, separators=(',', ': '))
    return render(request, 'kong_admin/show_config.html', {'config': config})
